﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using OnlineMetodist.API.Data;
using OnlineMetodist.API.Models;
using OnlineMetodist.API.Models.Request.Account;
using OnlineMetodist.API.Services;

namespace OnlineMetodist.API.Controllers
{
    [Route("api/[controller]")]
	[ApiController]
	public class AccountController : ControllerBase
	{
		private readonly UserManager<ApplicationUser> _userManager;
		private readonly RoleManager<IdentityRole> _roleManager;

		private readonly OnlineMetodistDbContext _context;
		private readonly TokenService _tokenService;
		private readonly ILogger<AccountController> _logger;

		public AccountController(UserManager<ApplicationUser> userManager, RoleManager<IdentityRole> roleManager, OnlineMetodistDbContext context, TokenService tokenService, ILogger<AccountController> logger)
		{
			_userManager = userManager;
			_roleManager = roleManager;
			_context = context;
			_tokenService = tokenService;
			_logger = logger;
		}


		[HttpPost]
		[Route("Register")]
		public async Task<IActionResult> Register(Registration request)
		{
			if (!ModelState.IsValid)
			{
				return BadRequest(ModelState);
			}

			try
			{
				var resultUser = await _userManager.CreateAsync(
				new ApplicationUser
				{
					UserName = request.Email,
					Email = request.Email
				},
					request.Password!
				);

				if (resultUser.Succeeded)
				{
					var user = await _userManager.FindByEmailAsync(request.Email);
					await _userManager.AddToRoleAsync(user, "PREPOD");
					request.Password = "";
					request.PasswordConfirm = "";
					Employee resultEmployee = new()
					{
						Surname = request.Surname,
						Name = request.Name,
						Patronymic = request.Patronymic,
						User = user
					};
					resultEmployee.UserId = resultEmployee.User.Id;
					_context.Employees.Add(resultEmployee);
					_context.SaveChanges();
					return CreatedAtAction(nameof(Register), new { email = request.Email }, request);
				}

				foreach (var error in resultUser.Errors)
				{
					ModelState.AddModelError(error.Code, error.Description);
				}

				return BadRequest(ModelState);

			}
			catch (Exception ex)
			{
				return BadRequest($"Failed to register {ex.Message}");
			}
		}


		[HttpPost]
		[Route("Login")]
		public async Task<IActionResult> Authenticate(Authorization request)
		{
			if (!ModelState.IsValid)
			{
				return BadRequest(ModelState);
			}

			var managedUser = await _userManager.Users.FirstOrDefaultAsync(x => x.Email == request.Email);
			//var managedUser = await _userManager.FindByEmailAsync(request.Email);
			if (managedUser == null)
			{
				return BadRequest("Invalid email");
			}

			var isPasswordValid = await _userManager.CheckPasswordAsync(managedUser, request.Password!);
			if (!isPasswordValid)
			{
				return BadRequest("Invalid password");
			}

			var userInDb = _context.Users.FirstOrDefault(u => u.Email == request.Email);

			if (userInDb is null)
			{
				return Unauthorized();
			}

			var accessToken = await _tokenService.CreateTokenAsync(userInDb);
			await _context.SaveChangesAsync();

			return Ok(accessToken);
		}

		[Authorize]
		[HttpPut]
		[Route("SelectMC")]
		public async Task<IActionResult> SetMethodicalCommision(string nameMC)
		{
			try
			{
				var methodicalCommission = await _context.MethodicaCommissions.FirstOrDefaultAsync(x => x.Name == nameMC);
				if (methodicalCommission == null)
				{
					return BadRequest("Methodical commission not found");

				}
				var user = await _userManager.FindByNameAsync(User.Identity.Name);
				if (user == null)
				{
					return BadRequest("User not found");
				}
				Employee employee = await _context.Employees.FirstOrDefaultAsync(x => x.UserId == user.Id);
				if (employee == null)
				{
					return BadRequest("Employee not found");
				}
				employee.MethodicalCommission = methodicalCommission;
				employee.IdMC = employee.MethodicalCommission.Id;
				await _context.SaveChangesAsync();
				return NoContent();
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside SelectMethodicalCommision action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
			
		}
	}
}
