﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using OnlineMetodist.API.Data;
using OnlineMetodist.API.Enums;
using OnlineMetodist.API.Models;
using OnlineMetodist.API.Models.Request.Events;
using OnlineMetodist.API.Models.TypeEvent;
using System.Text.Json;

namespace OnlineMetodist.API.Controllers
{
    [Authorize]
	[Route("api/[controller]/[action]")]
	[ApiController]
	public class EventsController : ControllerBase
	{
		private readonly OnlineMetodistDbContext _context;
		private readonly ILogger<EventsController> _logger;

		private readonly UserManager<ApplicationUser> _userManager;

		public EventsController(OnlineMetodistDbContext context, ILogger<EventsController> logger, UserManager<ApplicationUser> userManager)
		{
			_context = context;
			_logger = logger;
			_userManager = userManager;
		}

		

		[HttpGet]
		//[Route("GetEvents")]
		public async Task<ActionResult<IEnumerable<Event>>> GetEvent()
		{
			try
			{
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x => x.Employee).ThenInclude(e => e.Events)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				var events = applicationUser.Employee.Events;
				if (events == null)
				{
					_logger.LogError($"Events sent from client is null.");
					return NotFound("Events sent from client is null.");
				}
				return Ok(events);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside GetEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[HttpGet("{nameFormOfWork}")]
		//[Route("GetEvents")]
		public async Task<ActionResult<IEnumerable<Event>>> GetEvent(string nameFormOfWork)
		{
			if (!_context.FormsOfWorks.Any(x => x.Name.ToLower() == nameFormOfWork.ToLower()))
			{
				_logger.LogError($"Incorrect name form of work.");
				return BadRequest("Incorrect name form of work.");
			}
			try
			{
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x => x.Employee).ThenInclude(e => e.Events).ThenInclude( ev => ev.FormOfWorkFK)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				var events = applicationUser.Employee.Events.Where(x => x.FormOfWorkFK.Name.ToLower() == nameFormOfWork.ToLower());
				if (events == null)
				{
					_logger.LogInformation($"Events sent from client is null.");
					return NoContent();
				}
				return Ok(events);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside GetEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[HttpPost]
		//[Route("CreateHoldingEvent")]
		public async Task<ActionResult<Event>> CreateHoldingEvent(HoldingEvent model)
		{
			if (!ModelState.IsValid)
			{
				_logger.LogError("Invalid HoldingEvent object sent from client.");
				return BadRequest("Invalid model object");
			}

			try
			{
				FormOfWork form = await _context.FormsOfWorks.FirstAsync(x => x.Name == "Проведение");
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x=> x.Employee)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				Holding holdingEvent = new()
				{
					Name = model.Name,
					FormOfEvent = model.FormOfEvent,
					Location = model.Location,
					Status = model.Status,
					Result = model.Result
				};
				string specifications = JsonSerializer.Serialize(holdingEvent);
				Models.Event newEvent = new()
				{
					DateOfEvent = model.DateOfEvent,
					EndDateOfEvent = model.EndDateOfEvent,
					FormOfWorkFK = form,
					IdFormOfWork = form.Id,
					Specifications = specifications, 
					EmployeeId = applicationUser.Employee.Id,
					Employee = applicationUser.Employee
				};
				await _context.Events.AddAsync(newEvent);
				await _context.SaveChangesAsync();
				return CreatedAtAction("CreateHoldingEvent", new { id = newEvent.Id }, newEvent);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside CreateHoldingEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[HttpPost]
		//[Route("CreateParticipationEvent")]
		public async Task<ActionResult<Event>> CreateParticipationEvent(ParticipationEvent model)
		{
			if (!ModelState.IsValid)
			{
				_logger.LogError("Invalid ParticipationEvent object sent from client.");
				return BadRequest("Invalid model object");
			}

			try
			{
				FormOfWork form = await _context.FormsOfWorks.FirstAsync(x => x.Name == "Участие");
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x => x.Employee)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				Participation participationEvent = new()
				{
					Name = model.Name,
					FormOfParticipation = model.FormOfParticipation,
					FormOfEvent = model.FormOfEvent,
					Status = model.Status,
					Result = model.Result
				};
				string specifications = JsonSerializer.Serialize(participationEvent);
				Models.Event newEvent = new()
				{
					DateOfEvent = model.DateOfEvent,
					EndDateOfEvent = model.EndDateOfEvent,
					FormOfWorkFK = form,
					IdFormOfWork = form.Id,
					Specifications = specifications,
					EmployeeId = applicationUser.Employee.Id,
					Employee = applicationUser.Employee
				};
				await _context.Events.AddAsync(newEvent);
				await _context.SaveChangesAsync();
				return CreatedAtAction("CreateParticipationEvent", new { id = newEvent.Id }, newEvent);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside CreateParticipationEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[HttpPost]
		//[Route("CreateInternshipEvent")]
		public async Task<ActionResult<Event>> CreateInternshipEvent(InternshipEvent model)
		{
			if (!ModelState.IsValid)
			{
				_logger.LogError("Invalid InternshipEvent object sent from client.");
				return BadRequest("Invalid model object");
			}

			try
			{
				FormOfWork form = await _context.FormsOfWorks.FirstAsync(x => x.Name == "Стажировка");
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x => x.Employee)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				Internship internshipEvent = new()
				{
					Location = model.Location,
					QuantityOfHours = model.QuantityOfHours
				};
				string specifications = JsonSerializer.Serialize(internshipEvent);
				Models.Event newEvent = new()
				{
					DateOfEvent = model.DateOfEvent,
					EndDateOfEvent = model.EndDateOfEvent,
					FormOfWorkFK = form,
					IdFormOfWork = form.Id,
					Specifications = specifications,
					EmployeeId = applicationUser.Employee.Id,
					Employee = applicationUser.Employee
				};
				await _context.Events.AddAsync(newEvent);
				await _context.SaveChangesAsync();
				return CreatedAtAction("CreateInternshipEvent", new { id = newEvent.Id }, newEvent);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside CreateInternshipEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[HttpPost]
		//[Route("CreatePublicationEvent")]
		public async Task<ActionResult<Event>> CreatePublicationEvent(PublicationEvent model)
		{
			if (!ModelState.IsValid)
			{
				_logger.LogError("Invalid PublicationEvent object sent from client.");
				return BadRequest("Invalid model object");
			}

			try
			{
				FormOfWork form = await _context.FormsOfWorks.FirstAsync(x => x.Name == "Стажировка");
				ApplicationUser applicationUser = await _userManager.Users
					.Include(x => x.Employee)
					.FirstAsync(x => x.UserName == User.Identity.Name);
				PublicationEvent publicationEvent = new()
				{
					Type = model.Type,
					Name = model.Name,
					Place = model.Place
				};
				string specifications = JsonSerializer.Serialize(publicationEvent);
				Models.Event newEvent = new()
				{
					DateOfEvent = model.DateOfEvent,
					EndDateOfEvent = model.EndDateOfEvent,
					FormOfWorkFK = form,
					IdFormOfWork = form.Id,
					Specifications = specifications,
					EmployeeId = applicationUser.Employee.Id,
					Employee = applicationUser.Employee
				};
				await _context.Events.AddAsync(newEvent);
				await _context.SaveChangesAsync();
				return CreatedAtAction("CreatePublicationEvent", new { id = newEvent.Id }, newEvent);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside CreatePublicationEvent action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}
	}
}
