﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using OnlineMetodist.API.Data;
using OnlineMetodist.API.Models;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace OnlineMetodist.API.Controllers
{
	[Route("api/[controller]")]
	[Authorize(Roles = "Администратор")]
	[ApiController]
	public class ReportingPeriodController : ControllerBase
	{
		private readonly OnlineMetodistDbContext _context;
		private readonly ILogger<ReportingPeriodController> _logger;

		public ReportingPeriodController(OnlineMetodistDbContext context, ILogger<ReportingPeriodController> logger)
		{
			_context = context;
			_logger = logger;
		}

		[HttpGet]
		[Authorize]
		public async Task<ActionResult<IEnumerable<ReportingPeriod>>> GetReportingPeriod()
		{
			try
			{
				var reports = await _context.ReportingPeriods.ToListAsync();
				if (reports == null)
				{
					_logger.LogError($"Reports period object sent from client is null.");
					return NotFound("Role object sent from client is null.");
				}
				return Ok(reports);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside GetReportingPeriod action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[Authorize(Roles = "Заведующий корпусом")]
		[HttpPut("{id}")]
		public async Task<IActionResult> UpdateReportingPeriod(Guid id, ReportingPeriod reportingPeriod)
		{
			if (reportingPeriod is null)
			{
				_logger.LogError("ReportingPeriod object sent from client is null.");
				return BadRequest("ReportingPeriod object is null");
			}
			if (!ModelState.IsValid)
			{
				_logger.LogError("Invalid ReportingPeriod object sent from client.");
				return BadRequest("Invalid model object");
			}
			try
			{
				if (!ReportingPeriodExists(id))
				{
					_logger.LogError("Invalid id sent from client.");
					return BadRequest("Invalid id");
				}
				reportingPeriod.Id = id;
				_context.Entry(reportingPeriod).State = EntityState.Modified;
				await _context.SaveChangesAsync();
				return NoContent();
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside UpdateReportingPeriod action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[Authorize(Roles = "Заведующий корпусом")]
		[HttpPost]
		public async Task<ActionResult<ReportingPeriod>> CreateReportingPeriod(ReportingPeriod reportingPeriod)
		{
			try
			{
				if (reportingPeriod is null)
				{
					_logger.LogError("ReportingPeriod object sent from client is null.");
					return BadRequest("ReportingPeriod object is null");
				}
				if (!ModelState.IsValid)
				{
					_logger.LogError("Invalid ReportingPeriod object sent from client.");
					return BadRequest("Invalid model object");
				}
				reportingPeriod.Id = Guid.NewGuid();
				_context.ReportingPeriods.Add(reportingPeriod);
				await _context.SaveChangesAsync();

				return CreatedAtAction("GetReportingPeriod", new { id = reportingPeriod.Id }, reportingPeriod);
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside CreateReportingPeriod action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		[Authorize(Roles = "Заведующий корпусом")]
		[HttpDelete("{id}")]
		public async Task<IActionResult> DeleteReportingPeriod(Guid id)
		{
			try
			{
				var reportingPeriod = await _context.ReportingPeriods.FindAsync(id);
				if (reportingPeriod == null)
				{
					_logger.LogError($"ReportingPeriod with id: {id}, hasn't been found in db.");
					return NotFound();
				}

				_context.ReportingPeriods.Remove(reportingPeriod);
				await _context.SaveChangesAsync();

				return NoContent();
			}
			catch (Exception ex)
			{
				_logger.LogError($"Something went wrong inside DeleteReportingPeriod action: {ex.Message}");
				return StatusCode(500, "Internal server error");
			}
		}

		private bool ReportingPeriodExists(Guid id)
		{
			return _context.ReportingPeriods.Any(e => e.Id == id);
		}

	}
}
